Text

Struct Text 

pub struct Text<T>(pub T);
Map a SQL text value to/from a Rust type using Display and FromStr.

This can be useful for types that do not have a direct SQL equivalent, or are simply not supported by SQLx for one reason or another.

For strongly typed databases like Postgres, this will report the value’s type as TEXT. Explicit conversion may be necessary on the SQL side depending on the desired type.

Panics

You should only use this adapter with Display implementations that are infallible, otherwise you may encounter panics when attempting to bind a value.

This is because the design of the Encode trait assumes encoding is infallible, so there is no way to bubble up the error.

Fortunately, most Display implementations are infallible by convention anyway (the standard ToString trait also assumes this), but you may still want to audit the source code for any types you intend to use with this adapter, just to be safe.

Example: SocketAddr

MySQL and SQLite do not have a native SQL equivalent for SocketAddr, so if you want to store and retrieve instances of it, it makes sense to map it to TEXT:


use std::net::SocketAddr;

use sqlx::Connection;
use sqlx::mysql::MySqlConnection;
use sqlx::types::Text;

use uuid::Uuid;
use time::OffsetDateTime;

#[derive(sqlx::FromRow, Debug)]
struct Login {
    user_id: Uuid,
    socket_addr: Text<SocketAddr>,
    login_at: OffsetDateTime
}


let mut conn: MySqlConnection = MySqlConnection::connect("<DATABASE URL>").await?;

let user_id: Uuid = "e9a72cdc-d907-48d6-a488-c64a91fd063c".parse().unwrap();
let socket_addr: SocketAddr = "198.51.100.47:31790".parse().unwrap();

// CREATE TABLE user_login(user_id VARCHAR(36), socket_addr TEXT, login_at TIMESTAMP);
sqlx::query("INSERT INTO user_login(user_id, socket_addr, login_at) VALUES (?, ?, NOW())")
    .bind(user_id)
    .bind(Text(socket_addr))
    .execute(&mut conn)
    .await?;

let logins: Vec<Login> = sqlx::query_as("SELECT * FROM user_login")
    .fetch_all(&mut conn)
    .await?;

println!("Logins for user ID {user_id}: {logins:?}");

Tuple Fields

0: T

Implementations



impl<T> Text<T>

pub fn into_inner(self) -> T

Extract the inner value.